/*
 * Copyright (C) 2012-2019 Michael Hanrath <Michael.Hanrath@uni-koeln.de>
 *
 * This file is part of Diag2PS.
 *
 * Diag2PS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Diag2PS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Diag2PS.  If not, see <https://www.gnu.org/licenses/>.
 */

#pragma once


#include "Geometry/PosVector.H"
#include "DiagramElements.H"

#include <iostream>
#include <string>
#include <map>
#include <vector>
#include <stack>

namespace Diag2PS { // library namespaces $$$ begin

using namespace std;




class DiagramInterpreter {
public:
	struct PreprocessorExecutionFailed {};
	struct ParseErrors {};

	struct State {
		State() : location(0, 0) {}
		
	PosVector	location;			// current location
	map<std::string, PosVector>	storedLocations;
	};

	DiagramInterpreter(
		string const & tmpDir,
		string * const pConfigFile, 	// file included as prologue
		istream &, 						// Diag2PS input stream
		DiagramElements * de
	);


	struct NoSuchLabel {};
	struct DuplicateLabel {};
	struct EmptyStack {};

	void	move(string const & direction);		  	  	  	  	  	  	  	    // change current location (relative)
	void	move(PosVector const & location);	  	  	  	  	  	  	  	    // change current location (absolute)

	void	storepos(string const & label);										// store current location under "label"
	void	storepos(std::vector<std::pair<std::string, PosVector>> const &);
	PosVector	loadpos(string const & label);									// restore current location from "label"

	void	clearpos(std::vector<std::string> const & labels);					// clear corresponding locations, if labels is empty: clear all stored locations


	void	push();																// push current state onto stack
	void	pop();																// pop state from stack
	void	moveLastDirection();												// follow last direction
	void	storeDirection(PosVector const &);									// store last direction


	PosVector	location() const
	{	return _state.location;	}


	State const &	getState() const;
	DiagramInterpreter &	setState(State const &);
	
private:
DiagramElements * _de;

State	_state;
DirVector _lastDirection;

std::stack<std::pair<DiagramElements::State, State>>	_stateStack;
};
} // library namespaces $$$ end
