/*
 * Copyright (C) 2012-2019 Michael Hanrath <Michael.Hanrath@uni-koeln.de>
 *
 * This file is part of Diag2PS.
 *
 * Diag2PS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Diag2PS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Diag2PS.  If not, see <https://www.gnu.org/licenses/>.
 */


#include "Angle.H"
#include "DirVector.H"
#include <cmath>

namespace Diag2PS { // library namespaces $$$ begin


Angle::Angle(DirVector const & d) :
	_value(atan2(d.y, d.x))
{}

Angle::Angle(double w, AngleMeasure m) :
	_value(m==AngleMeasure::Radiant ? w : w*M_PI/180.0)
{}


double Angle::value(AngleMeasure m) const
{	return m==AngleMeasure::Radiant ? _value : _value/M_PI*180.0;	}

Angle & Angle::operator += (Angle const & a)
{	this->_value += a._value; return *this;	}

Angle operator + (Angle const & a, Angle const & b)
{	return Angle(a) += b;	}

} // library namespaces $$$ end
