/*
 * Copyright (C) 2012-2019 Michael Hanrath <Michael.Hanrath@uni-koeln.de>
 *
 * This file is part of Diag2PS.
 *
 * Diag2PS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Diag2PS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Diag2PS.  If not, see <https://www.gnu.org/licenses/>.
 */


#pragma once


#include <string>
#include <iostream>
#include <cmath>

#include "Vector.H"


namespace Diag2PS { // library namespaces $$$ begin

class PosVector;
class Angle;

struct DirVector : public Vector {

	explicit DirVector(Vector const &);
	DirVector(double x, double y, bool swap = false);
	
	DirVector(PosVector const & a, PosVector const & b);

	DirVector(std::string const & direction);

	DirVector & operator *= (double a);
	DirVector & operator += (DirVector const &);
	DirVector & operator -= (DirVector const &);

	DirVector perp() const;
	DirVector rotate(Angle const &) const;
	DirVector rotate(Angle const &, DirVector const & r) const;	// additionally scale by r

private:
	void update(double lx, double ly, int & n, double & mult);
};


} // library namespaces $$$ end
