/*
 * Copyright (C) 2012-2019 Michael Hanrath <Michael.Hanrath@uni-koeln.de>
 *
 * This file is part of Diag2PS.
 *
 * Diag2PS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Diag2PS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Diag2PS.  If not, see <https://www.gnu.org/licenses/>.
 */


#pragma once

#include <string>
#include <iostream>
#include <cmath>


namespace Diag2PS { // library namespaces $$$ begin


struct Vector {
	Vector(double x, double y, bool swap = false) :
		x(swap ? y : x), y(swap ? x : y) {}
	
	Vector(std::string const & direction);

	Vector & operator *= (double);
	Vector & operator += (Vector const &);
	Vector & operator -= (Vector const &);


	double norm2_sqr() const
	{	return x*x + y*y;	}

	double norm2() const
	{	return sqrt(norm2_sqr());	}


double	x, y;
};

std::ostream & operator << (std::ostream &, Vector const &);

} // library namespaces $$$ end
