/*
 * Copyright (C) 2012-2019 Michael Hanrath <Michael.Hanrath@uni-koeln.de>
 *
 * This file is part of Diag2PS.
 *
 * Diag2PS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Diag2PS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Diag2PS.  If not, see <https://www.gnu.org/licenses/>.
 */


#include "Label.H"

#include <iostream>
#include <vector>
#include <string>


namespace Diag2PS { // library namespaces $$$ begin


Label::Label(std::string const & positionOrientation, std::string const & label) : 
	position(Position::Center), orientation(0, 0), center(false), adjust(0), label(label)
{
using namespace std;
	for ( size_t i=1 ; i<positionOrientation.size() ; ++i )
		switch ( positionOrientation[i] ) {
			case 'B': position = Position::Begin;			break;
			case 'C': position = Position::Center;			break;
			case 'E': position = Position::End;				break;
			case 'l': orientation = DirVector(-1, 0);		break;
			case 'r': orientation = DirVector(+1, 0);		break;
			case 'u': orientation = DirVector(0, +1);		break;
			case 'd': orientation = DirVector(0, -1);		break;
			case 'c': center = true;						break;
			case '+': ++adjust;								break;
			case '-': --adjust;								break;
		}
}


std::ostream & operator << (std::ostream & o, Label const & l)
{
using namespace std;
static vector<string>	s1{"begin", "center", "end"};
	o << "[" 
		<< s1[static_cast<int>(l.position)] << ", "
		<< l.orientation << ", +-" << l.adjust << ", "
		<< "\"" << l.label << "\""
		<< "]";
	return o;
}



} // library namespaces $$$ end
