/*
 * Copyright (C) 2012-2019 Michael Hanrath <Michael.Hanrath@uni-koeln.de>
 *
 * This file is part of Diag2PS.
 *
 * Diag2PS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Diag2PS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Diag2PS.  If not, see <https://www.gnu.org/licenses/>.
 */


#pragma once

#include <string>
#include "Geometry/DirVector.H"

#include <iostream>

namespace Diag2PS { // library namespaces $$$ begin

enum class Position { Begin=0, Center=1, End=2 };

class Label {
public:
	Label() : position(Position::Center), orientation(0, 0), center(false), adjust(0) {}

	Label(Position position, DirVector orientation, bool center, std::string const & label) :
		position(position), orientation(orientation), center(center), adjust(0), label(label) {}

	Label(std::string const & positionOrientation, std::string const & label);

	operator bool () const
	{	return label.size()>0;	}
	
	
Position position;
DirVector orientation;
bool center;
int adjust;
std::string label;
};


std::ostream & operator << (std::ostream &, Label const &);

} // library namespaces $$$ end
