/*
 * Copyright (C) 2012-2019 Michael Hanrath <Michael.Hanrath@uni-koeln.de>
 *
 * This file is part of Diag2PS.
 *
 * Diag2PS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Diag2PS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Diag2PS.  If not, see <https://www.gnu.org/licenses/>.
 */



# include "Parser.H"
# include "Parser.hpp"

namespace Diag2PS { // library namespaces $$$ begin

istream * yySQDiagramsInputstream;

std::string currentLine;
std::string currentLineTmp;

Parser::Parser(
	Diag2PS::DiagramInterpreter & di, 
	Diag2PS::DiagramElements * de
	) : 
	trace_scanning (false), 
	trace_parsing (false), 
	errors(0),
	_diagramInterpreter(di), 
	_de(de)
{
}

Parser::~Parser ()
{
}

void
Parser::parse (istream & is)
{
  yySQDiagramsInputstream = &is;	// initialize global variable: very ugly
  scan_begin ();
  yyParser parser (*this);
  parser.set_debug_level (trace_parsing);
  parser.parse ();
  scan_end ();
}

void
Parser::error (const Diag2PS::location& l, const std::string& m)
{
  ++errors;
  std::cerr
    << "parse error in file \"" << *l.begin.filename << "\""
  	<< ", line " << l.begin.line << ", column " << l.begin.column << " - column " << l.end.column
	<< ":\n" << m << "\n"
	<< "current line:\n" << currentLine << std::endl;
}

void
Parser::error (const std::string& m)
{
  ++errors;
  std::cerr << m << std::endl;
}


}
// library namespaces $$$ end
