/*
 * Copyright (C) 2012-2019 Michael Hanrath <Michael.Hanrath@uni-koeln.de>
 *
 * This file is part of Diag2PS.
 *
 * Diag2PS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Diag2PS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Diag2PS.  If not, see <https://www.gnu.org/licenses/>.
 */


#pragma once

#include <string>
#include <map>
#include <iostream>

#include "../DiagramInterpreter.H"
#include "../DiagramElements.H"
#include "../Geometry/PosVector.H"
#include "../Geometry/DirVector.H"
#include "../Label.H"
#include <vector>

// library namespaces $$$ begin
using std::string;
using namespace std;

#ifndef YYSTYPE
union _YYSTYPE
{
std::string *sval;
Diag2PS::PosVector *	posVector;
Diag2PS::DirVector *	dirVector;
double *	real;
int * integer;
bool * yesno;
Diag2PS::Label * label;
std::vector<Diag2PS::Label> * labelSequence;
std::vector<std::string> * vString;
std::vector<std::pair<std::string, std::vector<Diag2PS::Label>>> * vLoopLabels;
Diag2PS::DiagramElements::LoopFragment * loopFragment;
Diag2PS::DiagramElements::ArrowDirection * arrowDir;
Diag2PS::DiagramElements::LineMultiplicity * lineMult;
};

//****************************************************************************************
struct RestrictedMode_StringTooLong {};
struct RestrictedMode_DisabledInstruction_PREAMBLE {};
struct RestrictedMode_DisabledInstruction_VERBATIM {};
struct RestrictedMode_DisabledInstruction_UPUT {};
struct RestrictedMode_DisabledInstruction_RPUT {};
//#define RESTRICTED_MODE // restricts string input for online version for security reasons
//****************************************************************************************

#define YYSTYPE _YYSTYPE
#endif

// Forward declarations.
namespace Diag2PS {

  class location;
  class yyParser;
class Parser;


extern istream * yySQDiagramsInputstream;

extern std::string currentLine;
extern std::string currentLineTmp;

class Parser
{
public:
	struct LexerUnexpectedCharacter {};

	Parser(
  		Diag2PS::DiagramInterpreter & di, 
		Diag2PS::DiagramElements * de
	);

  virtual ~Parser ();



  void scan_begin ();
  void scan_end ();

  void parse (istream &);

  void error (const Diag2PS::location& l, const std::string& m);
  void error (const std::string& m);



int result;
bool trace_scanning;
bool trace_parsing;
int	errors;

Diag2PS::DiagramInterpreter &	_diagramInterpreter;
Diag2PS::DiagramElements * _de;
};
} // library namespaces $$$ end


# define YY_DECL						 \
  int Diag2PSlex (YYSTYPE* yylval, Diag2PS::location* yylloc)
YY_DECL;





