/*
 * Copyright (C) 2012-2019 Michael Hanrath <Michael.Hanrath@uni-koeln.de>
 *
 * This file is part of Diag2PS.
 *
 * Diag2PS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Diag2PS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Diag2PS.  If not, see <https://www.gnu.org/licenses/>.
 */


%{
# include <cstdlib>
# include <errno.h>
# include <limits.h>
# include <string>
//#include "Parser.y.H"
# include "Parser.H"
# include "Parser.hpp"
using namespace Diag2PS;
#define NS Diag2PS::
#define NS2 NS yyParser::token::
%}

%option noyywrap nounput batch debug

/******************************/
%option prefix="Diag2PSlex"
/******************************/


blank		[ \t]
alpha   	[_A-Za-z]
dig     	[0-9]
direction	[lLrRuUdD.|]+
name    	\"[^"]*\"
float_num   [-+]?{dig}+\.?{dig}*([eE][-+]?{dig}+)?
operators	[<>=(),+-[\]:]
label		@[BEC]?[udlrc]?[-+]*

%{
# define YY_USER_ACTION  yylloc->columns (yyleng);
#define YY_INPUT(buf,result,max_size) \
{ \
int c = NS yySQDiagramsInputstream->get(); \
currentLine = currentLineTmp;\
currentLineTmp += std::string(1, static_cast<char>(c));\
if ( c=='\n' ) currentLineTmp = "";\
result = (c == EOF) ? YY_NULL : (buf[0] = c, 1); \
}
%}
%%
%{
  yylloc->step ();
%}
{blank}+   		yylloc->step ();
;+				{ yylloc->step (); return NS2 SEPARATOR;	}
\n				{ yylloc->lines (yyleng); yylloc->step (); return NS2 SEPARATOR;	}
~				{ yylloc->step (); return NS2 MOVELASTDIRECTION;	}

\/\/.*			{ yylloc->lines (yyleng);  }		/* comment: ignore rest of line */

#					{ /* read preprocessor infos */
					char c = 0; 
					std::string line; 
						while ( (c=NS yySQDiagramsInputstream->get())!='\n' )
							line += c;
					std::size_t i1 = line.find("\"");
					std::size_t i2 = line.find("\"", i1+1);
					
						if ( i1!=string::npos  &&  i2!=string::npos )
						{
							++i1;
						std::string filename(line.substr(i1, i2-i1));
							if ( filename.find("TMP.Diag2PS.")!=string::npos )
								filename = "<stdin>";

						std::istringstream is(line);
						int lineNo;
							is >> lineNo;
							--lineNo;
 /*							std::cerr << "::: " << line << "  :  " << filename << ", " << lineNo << std::endl;*/
							delete yylloc->begin.filename;
							*yylloc = location(new std::string(filename), lineNo);
							yylloc->step ();
						}
						currentLineTmp = "";
					}


yes					{ return NS2 KEYWORD_yes;	}
no					{ return NS2 KEYWORD_no;	}

on					{ return NS2 KEYWORD_yes;	}
off					{ return NS2 KEYWORD_no;	}

move				{ return NS2 KEYWORD_move;	}
storepos			{ return NS2 KEYWORD_storepos;	}
clearpos			{ return NS2 KEYWORD_clearpos;	}
push				{ return NS2 KEYWORD_push;	}
pop					{ return NS2 KEYWORD_pop;	}

cross				{ return NS2 KEYWORD_cross;	}
bullet				{ return NS2 KEYWORD_bullet;	}
bbullet				{ return NS2 KEYWORD_bbullet;	}
circle				{ return NS2 KEYWORD_circle;	}
bcircle				{ return NS2 KEYWORD_bcircle;	}
dc					{ return NS2 KEYWORD_dc;	}
square				{ return NS2 KEYWORD_square;	}
bsquare				{ return NS2 KEYWORD_bsquare;	}

cl					{ return NS2 KEYWORD_cl;	}
clo					{ return NS2 KEYWORD_clo;	}
il					{ return NS2 KEYWORD_il;	}
id					{ return NS2 KEYWORD_id;	}
pl					{ return NS2 KEYWORD_pl;	}
sl					{ return NS2 KEYWORD_sl;	}

phl					{ return NS2 KEYWORD_phl;	}
loop				{ return NS2 KEYWORD_loop;	}

reverse				{ return NS2 KEYWORD_reverse;	}
snake				{ return NS2 KEYWORD_snake;	}
bubble				{ return NS2 KEYWORD_bubble;	}

pointer				{ return NS2 KEYWORD_pointer;	}

preamble			{
#ifdef RESTRICTED_MODE
						std::cerr << "keyword \"preamble\" disabled" << endl;
						throw RestrictedMode_DisabledInstruction_PREAMBLE();
#endif
					char c = 0; 
						yylval->sval = new std::string(); 
						while ( (c=NS yySQDiagramsInputstream->get())!='\n' )
							*(yylval->sval) += c;
					size_t i = (yylval->sval)->find("mitabrev");
						if ( i!=std::string::npos )
						{
							(yylval->sval)->erase(i, 8);
							return NS2 VERBATIM;
						}
						yySQDiagramsInputstream->unget();
						return NS2 PREAMBLE;
					}

verbatim			{
#ifdef RESTRICTED_MODE
						std::cerr << "keyword \"verbatim\" disabled" << endl;
						throw RestrictedMode_DisabledInstruction_VERBATIM();
#endif
					char c = 0; 
						yylval->sval = new std::string(); 
						while ( (c=NS yySQDiagramsInputstream->get())!='\n' )
						{
							*(yylval->sval) += c;
						size_t i = (yylval->sval)->find("mitabrev");
							if ( i!=std::string::npos )
							{
								(yylval->sval)->erase(i, 8);
								return NS2 VERBATIM;
							}
						}
						yySQDiagramsInputstream->unget();
						return NS2 VERBATIM;
					}

rput				{
#ifdef RESTRICTED_MODE
						std::cerr << "keyword \"rput\" disabled" << endl;
						throw RestrictedMode_DisabledInstruction_RPUT();
#endif
					char c = 0; 
					char oc = 0; 
						yylval->sval = new std::string(); 
						while ( (c=NS yySQDiagramsInputstream->get())!='\n' )
						{
							if ( c==';' || c==')' )
							{
								if ( oc=='\\' )
									(yylval->sval)->erase(yylval->sval->length()-1, 1);
								else
									break;
							}
							*(yylval->sval) += c;
							oc = c;
						}
						yySQDiagramsInputstream->unget();
						return NS2 RPUT;
					}

uput				{
#ifdef RESTRICTED_MODE
						std::cerr << "keyword \"uput\" disabled" << endl;
						throw RestrictedMode_DisabledInstruction_UPUT();
#endif
					char c = 0; 
					char oc = 0; 
						yylval->sval = new std::string(); 
						while ( (c=NS yySQDiagramsInputstream->get())!='\n' )
						{
							if ( c==';' || c==')' )
							{
								if ( oc=='\\' )
									(yylval->sval)->erase(yylval->sval->length()-1, 1);
								else
									break;
							}
							*(yylval->sval) += c;
							oc = c;
						}
						yySQDiagramsInputstream->unget();
						return NS2 UPUT;
					}

{label}				{
						yylval->sval = new std::string(yytext); 
						return NS2 KEYWORD_label;
					}
					
					
showGrid			{ return NS2 KEYWORD_showGrid;	}

color				{ return NS2 KEYWORD_color;	}
unitSize			{ return NS2 KEYWORD_unitSize;	}
lineWidth			{ return NS2 KEYWORD_lineWidth;	}
path				{ return NS2 KEYWORD_path;	}
arrowWidth			{ return NS2 KEYWORD_arrowWidth;	}
arrowLength2Width	{ return NS2 KEYWORD_arrowLength2Width;	}
arrowFillColor		{ return NS2 KEYWORD_arrowFillColor;	}
bentArrows			{ return NS2 KEYWORD_bentArrows;	}
BBXCenter			{ return NS2 KEYWORD_BBXCenter;	}
BBYCenter			{ return NS2 KEYWORD_BBYCenter;	}
labelYShift			{ return NS2 KEYWORD_labelYShift;	}
labelDist			{ return NS2 KEYWORD_labelDist;	}
pointDist			{ return NS2 KEYWORD_pointDist;	}

{direction}		yylval->sval = new std::string(yytext); return NS2 DIRECTION;

{float_num}		yylval->sval = new std::string(yytext); return NS2 FLOAT_NUM;

{name}			yylval->sval = new std::string(yytext); *yylval->sval = yylval->sval->substr(1, yylval->sval->length()-2); return NS2 NAME;

{operators}		{ return yytext[0]; }

.				{ cerr << "Lexer error: \n"
					<< "file \"" << *yylloc->begin.filename << "\", line " << yylloc->begin.line << ", column " << yylloc->begin.column << "\n"
					<< "unexpected character \"" << yytext[0] << "\""  
					<< "\ncurrent line:\n" << currentLine << endl;
					throw Parser::LexerUnexpectedCharacter(); }

%%

void
NS Parser::scan_begin ()
{
  yy_flex_debug = trace_scanning;
//  if (!(yyin = fopen (file.c_str (), "r")))
//    error (std::string ("cannot open ") + file);
}

void
NS Parser::scan_end ()
{
//  fclose (yyin);
}
