/*
 * Copyright (C) 2012-2019 Michael Hanrath <Michael.Hanrath@uni-koeln.de>
 *
 * This file is part of Diag2PS.
 *
 * Diag2PS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Diag2PS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Diag2PS.  If not, see <https://www.gnu.org/licenses/>.
 */


#pragma once


#include "../DiagramElements.H"

#include <array>
#include <string>


namespace Diag2PS { // library namespaces $$$ begin

using namespace std;




class DiagramElements_PSTricks : 
	public DiagramElements {
public:
	struct LaTeXRunFailed {};
	struct DviPSRunFailed {};
	struct PDFcropRunFailed {};
	struct PDF2CairoRunFailed {};

	struct OutputTypes { unsigned int tex:1, ps:1, pdf:1, svg:1; };	// will end in .tex, .ps, .pdf, .svg

	DiagramElements_PSTricks(
		string const & tmpDir,
		string const & outputFilename,
		OutputTypes outputType = {1, 0, 0, 0},
		double unitSize = 1.0, // height of loop
		double lineWidth = 1.0,
		double arrowSize = 1.0
		);


	virtual void	verbatim(const string & s);	
	virtual void	rput(PosVector const & loc, const string & s);	
	virtual void	rput(PosVector const & loc, string const & rputOptions, const string & s);	
	virtual void	uput(PosVector const & loc, const string & s);	
	virtual void	uput(PosVector const & loc, string const & uputOptions, const string & s);	
	virtual void	pointer(PosVector const & loc1, PosVector const & loc2, std::vector<Label> const & labels);


	virtual void	phLine(
		PosVector const & loc1, 
		PosVector const & loc2, 
		ArrowDirection arrowDirection,
		double arrowShift,
		LineMultiplicity lineMultiplicity,
		std::vector<Label> const &
		);

	virtual void	bubble(
		const PosVector & a, 
		double loopWidth,
		ArrowDirection arrowDirection, 
		LineMultiplicity lineMultiplicity,
		std::vector<Label> const & labels
		);

	virtual void	snake(
		const PosVector & loc1, 
		LoopFragment loopFragment, 
		const PosVector & loc2, 
		double loopWidth,
		ArrowDirection arrowDirection, 
		double arrowShift,
		LineMultiplicity lineMultiplicity
		);

	virtual void	interactionLine(PosVector const & loc1, PosVector const & loc2);
	virtual void	interactionDots(PosVector const & loc1, PosVector const & loc2);
	virtual void	photonLine(const PosVector & a, const PosVector & b);
	virtual void	springLine(const PosVector & a, const PosVector & b);

	virtual void	clusterLine(PosVector const & loc1, PosVector const & loc2);
	virtual void	clusterLineOpen(PosVector const & loc1, PosVector const & loc2);

	virtual void	doSetlineWidth(double lineWidth);
	virtual void	cross(PosVector const & loc);
	virtual void	bullet(PosVector const & loc);
	virtual void	bbullet(PosVector const & loc);
	virtual void	circle(PosVector const & loc);
	virtual void	bcircle(PosVector const & loc);
	virtual void	disconnectingCircle(PosVector const & loc);
	virtual void	square(PosVector const & loc);
	virtual void	bsquare(PosVector const & loc);

	virtual void draw() const;


protected:
	virtual std::vector<PosVector>	doLoopFragment(
		PosVector const & loc1, 
		LoopFragment loopFragment, 
		PosVector const & loc2, 
		double loopWidth,
		ArrowDirection arrowDirection,
		double arrowShift,
		LineMultiplicity lineMultiplicity,
		std::vector<std::pair<std::string, std::vector<Label>>> const & yBranches
	);

	void	arrowOnLine(
		PosVector const & loc1, 
		PosVector const & loc2, 
		ArrowDirection arrowDirection,
		double arrowShift,
		LineMultiplicity lineMultiplicity
		);

	void	arrowOnLine(
		PosVector const & loc, 
		Angle angle, 
		ArrowDirection arrowDirection,
		double arrowShift,
		LineMultiplicity lineMultiplicity
		);

	void	arrowOnEllipticArc(
		PosVector const & center, 
		DirVector r, 
		Angle angle,
		bool forward,
		LineMultiplicity lineMultiplicity,
		bool onEllipsis
		);

	void	arrowOnEllipticArc(
		PosVector const & center, 
		DirVector r, 
		Angle angle,
		ArrowDirection arrowDirection,
		LineMultiplicity lineMultiplicity,
		bool onEllipsis
		);


private:
	void	putLabel(
		Label const & label, 
		std::array<PosVector, 3> const & bce, 
		DirVector dir,
		bool isPHLine = true
	);
	
	static double sqr(double x)	{	return x*x;	}

string _tmpDir;
string _outputFilename;
OutputTypes _outputType;
};


} // library namespaces $$$ end
