/*
 * Copyright (C) 2012-2019 Michael Hanrath <Michael.Hanrath@uni-koeln.de>
 *
 * This file is part of Diag2PS.
 *
 * Diag2PS is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Diag2PS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Diag2PS.  If not, see <https://www.gnu.org/licenses/>.
 */


#include "../DiagramElements_PSTricks.H"
#include "../../DiagramInterpreter.H"

#include <fstream>
#include <stdlib.h>
#include <sstream>
#include <string>
#include <filesystem>
#include <unistd.h>


using namespace Diag2PS;
using namespace std;;


static string tmpDir;


void cleanup()
{
using namespace std::filesystem;
path p (tmpDir);
	if ( exists(p) && is_directory(p) ) 
		remove_all(p);
}





int	main(int argc, char const * const * argv)
{
//char	tmpin[] = "/var/WWW_Diag2PS/Diag2PS_TMP.XXXXXX";
char	tmpin[] = "/tmp/Diag2PS_TMP.XXXXXX";
	mkdtemp(tmpin);
	tmpDir = std::string(tmpin);

	atexit(cleanup);
	
char * const pEnv = getenv("HOME");
string configFile(string(pEnv==nullptr ? "" : pEnv)+"/.Diag2PS");
string *	pConfigFile = (std::filesystem::is_regular_file(std::filesystem::path(configFile)) ? &configFile : nullptr);
DiagramElements_PSTricks::OutputTypes	outputType = { 0, 0, 0, 0 };
string	filename;
istream * pis = &cin;
ifstream input;
	for ( int i=1 ; i<argc ; ++i )
	{
	std::string	arg(argv[i]);
		if (  arg == "-tex" )
			outputType.tex = 1;
		else {
		if ( arg == "-ps" )
			outputType.ps = 1;
		else {
		if ( arg == "-pdf" )
			outputType.pdf = 1;
		else {
		if ( arg == "-svg" )
			outputType.svg = 1;
		else {
		if ( arg == "-noconfig" )
			pConfigFile = nullptr;
		else {
		if ( arg == "-config" )
		{
			if ( argc<i+2 )
			{
				cerr << "Diag2PS: missing config file" << endl;
				throw;
			}
			configFile = argv[++i];
			pConfigFile = &configFile;
		}
		else {
		if ( arg[0] == '-' )
		{
			cerr << "Diag2PS: unknown option \"" << argv[i] << "\"" << endl;
			throw;
		}
		else {
			filename = arg;
			input = ifstream(filename);
			if ( !input )
			{
				cerr << "Diag2PS: can not open file \"" << arg << "\"" << endl;
				throw;
			}
			pis = &input;
			
		}}}}}}}
	}

	if ( !(outputType.tex || outputType.ps || outputType.pdf || outputType.svg) )
		outputType.tex = 1;
	if ( (outputType.ps || outputType.pdf || outputType.svg)  &&  filename.size()==0 )
	{
		cerr << "Diag2PS: flags -ps, -pdf, -svg require filename" << endl;
		throw;
	}
		
	
	try {
	DiagramElements_PSTricks	ps(tmpDir, filename.substr(0, filename.rfind('.')), outputType, 1, 1.5, 5.0);
	DiagramInterpreter	m(tmpDir, pConfigFile, *pis, &ps);
	}
	catch ( ... )
	{
		cleanup();
		rethrow_exception(current_exception());
	}

}
